<?php

namespace Teknisa\DF\Modules\Entrada\Controllers;

use Teknisa\DF\Modules\Entrada\Services\Dfe18500DocumentosParaComposicaoDeCustosService;

use Zeedhi\Framework\DTO\Request;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DataSource;
use \Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Estoque\Entities\Lanctoestoq;
use Geral\Entities\Itemproduto;
use Fiscal\Entities\Notafiscal;

class Dfe18500DocumentosParaComposicaoDeCustosController extends \Teknisa\DF\Controllers\TeknisaCrud {
    
    protected $documentosParaComposicaoDeCustosService;
    protected $entityManager; 
    protected $crudService;
    protected $parameterBag;

    public function __construct($crudService, $parameterBag, Dfe18500DocumentosParaComposicaoDeCustosService $documentosParaComposicaoDeCustosService, EntityManager $entityManager, $environment) {
        $this->documentosParaComposicaoDeCustosService =  $documentosParaComposicaoDeCustosService;
        $this->crudService = $crudService;
        $this->entityManager = $entityManager;
        $this->environment   = $environment;
    }
    
    public function documentosComposicaoDeCustos(Request\Filter $request, Response $response) {
        $filterCriteria = $request->getFilterCriteria();
        $conditions = $filterCriteria->getConditions();
        
        //$conditions[0] --> P_CDFORNECED e $conditions[1] --> P_TCDFORNECED
        if(empty($conditions[0]["value"])) {
            $conditions[1]["value"] = 'TD';
        } else {
            $conditions[1]["value"] = '';
        }
        
        //$conditions[2] --> P_DATAINI, $conditions[3] --> P_DATAFIN  e  $conditions[4] --> P_TDATA
        if(empty($conditions[2]["value"]) && empty($conditions[3]["value"])) {
            $conditions[4]["value"] = 'TD';
        } else {
            $conditions[4]["value"] = '';
        }

        //$conditions[5] --> P_IDSTATUS e $conditions[6] --> P_IDSTATUSNFE
        if($conditions[5]["value"] === 'E') {
            $conditions[6]["value"] = 'A';
            $conditions[5]["value"] = 'TD';
        } else if(empty($conditions[5]["value"])) {
            $conditions[5]["value"] = 'TD';
            $conditions[6]["value"] = 'TD';
        } else {
            $conditions[6]["value"] = 'TD';
        }
       
        $SdataSourceName = 'documentos_composicao_de_custos';
        $dataSet = $this->crudService->getFilteredDataSet('documentos_composicao_de_custos', $conditions, $filterCriteria->getPage(), $filterCriteria->getPageSize());
    
        $dataSet = new DataSet('documentos_composicao_de_custos', $dataSet->getRows());
        $response->setDataSets(array($dataSet));
    }
    
    public function salvaRateioIt(Request\DataSet $request, Response $response) {
        try {
            $rows = $request->getDataSet()->getRows();
            $vrtotalrateio = round($this->documentosParaComposicaoDeCustosService->getVrRateioCustoItTotal($rows[0]["CDFILIAL"], $rows[0]["NRLANCTONF"]), 2);
            $vrtotalcustos = 0;

            foreach($rows as $row) {
                $vrtotalcustos = $vrtotalcustos + $row["VRRATEIO"];
            }
            
            if((round($vrtotalcustos, 2) - $vrtotalrateio) == 0)  {  
                foreach($rows as $row) {
                    $this->documentosParaComposicaoDeCustosService->updateRateioit($row["CDFILIAL"], $row["NRLANCTONF"], $row["NRORG"], $row["NRSEQITPROD"], $row["VRRATEIO"]);
                }
            } else {
                throw new \Exception ("ERRO_SALVAR_RATEIO_CUSTOS"); 
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function confirmarComposicao(Request\Row $request, Response $response) {
        try {
            $row = $request->getRow();
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            $lanctoestoq = Lanctoestoq::findOneByCdfilialAndNrlanctonfAndNrorg($row["CDFILIAL"], $row["NRLANCTONF"], $row["NRORG"]);
            $this->documentosParaComposicaoDeCustosService->callProcedureConhecimentoTransporte($row["CDFILIAL"], $row["NRLANCTONF"], $lanctoestoq->getNrlancestq());
            
            $connection->commit();
            $this->entityManager->flush();
            
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
            $connection->rollback();
        }
    }
    
    public function salvarDocumento(Request\DataSet $request, Response $response) {
        try {              
            $rows = $request->getDataSet()->getRows();

            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            foreach($rows as $row) {
                $this->documentosParaComposicaoDeCustosService->insereRelacratcustonf($row["CDFILIAL"], $row["NRLANCTONF"], $row["CDFILIDOCCOMP"], $row["NRLANCTODOCCOMP"]); 
            }
            
            $this->documentosParaComposicaoDeCustosService->calculaValorTotalCustos($rows[0]["CDFILIAL"], $rows[0]["NRLANCTONF"], $rows[0]["NRORG"]);
            
            $connection->commit();
            $this->entityManager->flush();
            
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
            $connection->rollback();

        }
    }
    
    public function excluirDocumento(Request\DataSet $request, Response $response) {
        try {
            $rows = $request->getDataSet()->getRows();
            foreach($rows as $row) {
                
                $this->documentosParaComposicaoDeCustosService->deleteRelacratcustonf($row["CDFILIAL"], $row["NRLANCTONF"], $row["CDFILIDOCCOMP"], $row["NRLANCTODOCCOMP"]);
            }
            
            $this->documentosParaComposicaoDeCustosService->calculaValorTotalCustos($rows[0]["CDFILIAL"], $rows[0]["NRLANCTONF"], $rows[0]["NRORG"]);
            $this->documentosParaComposicaoDeCustosService->alteraStatusNotafiscal($rows[0]["CDFILIAL"], $rows[0]["NRLANCTONF"], $rows[0]["NRORG"],  'D');
            
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
        
    }
}