<?php

namespace Teknisa\DF\Modules\Saida\Services;

use Teknisa\DF\Modules\Saida\Factories\Grs15000ExportacaoNfFaturamentoSapFactory;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\Libs\Util\Masks\Masks;
use Teknisa\Libs\Util\Environment;
use Doctrine\ORM\EntityManager;
use Teknisa\Libs\Util\DateUtil;

class Grs15000ExportacaoNfFaturamentoSapService {
    protected $dataSourceManager;
    protected $entityManager;
    protected $crudService;
    protected $environment;
    protected $factory;
    protected $masks;

    public function __construct($crudService, $dataSourceManager, \Doctrine\ORM\EntityManager $entityManager, Environment $environment, Grs15000ExportacaoNfFaturamentoSapFactory $factory, Masks $masks) {
        $this->crudService              = $crudService;
        $this->dataSourceManager = $dataSourceManager;
        $this->entityManager     = $entityManager;
        $this->environment       = $environment;
        $this->factory           = $factory;
        $this->masks             = $masks;
    }
    
    public function exportarNf($rows) {
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();
            
            foreach ($rows as $row) {
                $this->factory->procedureExportacao($row['CDFILIAL'], $row['NRLANCTONF']);
                $this->entityManager->flush();
                
                $this->factory->updateNotaFiscal($row['CDFILIAL'], $row['NRLANCTONF']);
                $this->entityManager->flush();
                
                if($row['IDAVISO']==='S'){
                    $this->factory->updateAviso($row['IDAVISO'], $row['CRCREDITO'], $row['DSOBSERVCRED'], $row['CDUNICOAVISO'], $row['CDUNICO']);
                    $this->entityManager->flush();
                
                    // Se a nota possuir aviso, tambem sera exportada a nota de credito relacionada a mesma
                    $this->factory->procedureExportacao(substr($row['CDUNICOAVISO'],0,4 ), substr($row['CDUNICOAVISO'], 4, 10) );
                    $this->entityManager->flush();
                }

            }
                
            $connection->commit();
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }   
    
    public function aplicarMascaraInscricao($rows,$nomeCampo) {
        foreach ($rows as &$row) {
            if (strlen($row[$nomeCampo]) == 14) {
                $row[$nomeCampo] = $this->masks->mask($row[$nomeCampo], '##.###.###/####-##');
            }
            else if (strlen($row[$nomeCampo])==11) {
                $row[$nomeCampo] = $this->masks->mask($row[$nomeCampo], '###.###.###-##');
            }
        }
        return $rows;
    }

    public function salvarNotaCredito($row) {
        try {
            $connection = $this->entityManager->getConnection();
            $connection->beginTransaction();

            $this->factory->salvarNotaCredito($row['CDUNICOAVISO'], $row['CDUNICO']);
            $this->entityManager->flush();

            $connection->commit();
        } catch (\Exception $e) {
            $this->entityManager->rollback();
            throw $e;
        }
    }   
    
    public function getFilterExportSap() {
        return $this->factory->getFilterExportSap($this->environment->getCdFilial());
    }
}