<?php
namespace Teknisa\DF\Services;

use Zeedhi\Framework\DataSource\FilterCriteria;
use Teknisa\Libs\Util\Environment;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;

use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;

class Ger82000CodigoTributacaoMunicipioService extends \Teknisa\DF\Controllers\TeknisaCrud {

	protected $environment;
	protected $dataSourceManager;
	protected $newCode;
	protected $factory;

    public function __construct(Environment $environment, $dataSourceManager, $newCode, $factory){
        $this->environment = $environment;
        $this->dataSourceManager = $dataSourceManager;
        $this->newCode = $newCode;
        $this->factory = $factory;
    }

    public function cadastroAutomaticoCodTribMuni($rowObject) {
        $row = (array) $rowObject;
        $dataSourceNameToSave = 'codtribprodpadr';
        $produtos = $this->getProdutos($row['CDPRODUTOINI'], $row['CDPRODUTOFIN']);
        $rowsUpdate = array();
        foreach ($produtos as $prod) {
            $produto = array_merge($prod, $row);
            unset($produto['__rightActions']);
            unset($produto['__leftActions']);
            unset($produto['P_CDPRODINI']);
            unset($produto['P_CDPRODFIN']);
            $produto['CDEMPRESA'] = $row['CDEMPRESA'];
            $produto['CDINSCMUNI'] = $row['CDINSCMUNI'];
            $produto['CDTRIBPRODPADR'] = (isset($row['CDTRIBPRODPADR'])) ? $row['CDTRIBPRODPADR'] : '';
            $produto['CDCLASSERVPADR'] = (isset($row['CDCLASSERVPADR'])) ? $row['CDCLASSERVPADR'] : '';
            $produto['DSCLASSERVPADR'] = (isset($row['DSCLASSERVPADR'])) ? $row['DSCLASSERVPADR'] : '';
            $produto['__is_new'] = true;
            array_push($rowsUpdate, $produto);
        }
        $this->deleteCodtribprodpadr($row['CDPRODUTOINI'], $row['CDPRODUTOFIN'], $row['CDEMPRESA'], $row['CDINSCMUNI']);
        $dataSet = new DataSet($dataSourceNameToSave, $rowsUpdate);
        $this->dataSourceManager->persist($dataSet);
    }
    
    protected function getProdutos($P_CDPRODINI, $P_CDPRODFIN) {
        $conditions =  array();
        $nvProduto  = $this->factory->getNvProduto();
        $nivel = substr_count($nvProduto, ".") + 1;
        $filterCriteria = new FilterCriteria('produto_cadauto', $conditions);
        $paramsProdutos = $params = array(
                    ':P_CDPRODFIN'=>$P_CDPRODFIN,
                    ':P_CDPRODINI'=>$P_CDPRODINI,
                    ':P_NVPRODUTO'=>$nivel
                );
        $produtos = $this->factory->getProdutos($params);
        $prodReturn = array();
        foreach ($produtos as $prod) {
            $produtoCadAuto = $this->adicionaCampos($prod);
            array_push($prodReturn, $produtoCadAuto);
        }
        return $prodReturn;
    }
    
    protected function deleteCodtribprodpadr($P_CDPRODINI, $P_CDPRODFIN, $P_CDEMPRESA, $P_CDINSCMUNI) {
        $conditions =  array();
        $filterCriteria = new FilterCriteria('codtribprodpadr', $conditions);
        $nvProduto  = $this->factory->getNvProduto();
        $nivel = substr_count($nvProduto, ".") + 1;
        $paramsProdutos = $params = array(
                    ':P_CDPRODINI'=>$P_CDPRODINI,
                    ':P_CDPRODFIN'=>$P_CDPRODFIN,
                    ':P_CDEMPRESA'=>$P_CDEMPRESA,
                    ':P_CDINSCMUNI'=>$P_CDINSCMUNI
                );
        $this->factory->deleteCodtribprodpadr($params);
    }
    
    protected function adicionaCampos($row) {
        $nrorg = $this->environment->getNrorgTrab();
		$operador = $this->environment->getCdOperador();
		$currentTime = $this->environment->getCurrentTime()->format("d/m/Y H:i:s");
		$row['NRORG'] = $nrorg;
		$row['DTINCLUSAO'] = $currentTime;
		$row['DTULTATU'] = $currentTime;
		$row['CDOPERINCLUSAO'] = $operador;
		$row['CDOPERULTATU'] = $operador;
		$row['IDATIVO'] = 'S';
		$row['NRORGINCLUSAO'] = $nrorg;
		$row['NRORGULTATU'] = $nrorg;
		$row['__is_new'] = false;
		return $row;
    }

}